#!/usr/bin/env php
<?php
set_include_path(get_include_path() . PATH_SEPARATOR . '/usr/local/mgr5/include/php');
define('__MODULE__', 'fraudhosting');
require_once 'fraudhosting_func.php';

Info("FraudHosting plugin started");

// Define config files
$moduleConfig = parseConfig('etc/fh_module.conf');
$dbConfig = parseConfig('etc/billmgr.conf.d/db.conf');

// Get env params
$param_func = getenv('PARAM_func');
$param_elid = getenv('PARAM_elid');
$param_out = getenv('PARAM_out');
$param_authlevel = getenv('AUTH_LEVEL');
$param_authuser = getenv('AUTH_USER');
$param_cookie = getenv('HTTP_COOKIE');

// Stop, if non matched params
if ($param_authlevel !== '29') { die("Auth level not admin"); }
if ($param_func !== 'fraudhosting.check') { die("Requested function is wrong"); }

try {
    $pdo = connectToDatabase($dbConfig['DBHost'], $dbConfig['DBUser'], $dbConfig['DBPassword'], $dbConfig['DBName']);
} catch (PDOException $e) {
    if ($dbConfig['DBHost'] === 'localhost') {
        try {
            $pdo = connectToDatabase('127.0.0.1', $dbConfig['DBUser'], $dbConfig['DBPassword'], $dbConfig['DBName']);
        } catch (PDOException $e) {
            Error("Database connection failed: " . $e->getMessage());
            die("Unable to connect to the database.");
        }
    } else {
        Error("Database connection failed: " . $e->getMessage());
        die("Unable to connect to the database.");
    }
}

Info("Auth user: " . $param_authuser);
Info("Auth level: " . $param_authlevel);
Info("Requested check for client ID: " . $param_elid);

if ($param_out == 'text') { $check_type = 'auto'; } else { $check_type = 'manual'; }
Info("Requested check type: " . $check_type);

// Getting client register IP and create hash
$ip_query = "SELECT registration_ip FROM account WHERE id = :id";
$stmt = $pdo->prepare($ip_query);
$stmt->bindParam(':id', $param_elid, PDO::PARAM_INT);
$stmt->execute();
$ip_result = $stmt->fetchColumn();
$hashed_ip = calc_hash(mb_strtolower(str_replace(" ", "", $ip_result), 'UTF-8'));

// Getting first user email for client and create hash
$email_query = "SELECT email FROM user WHERE account = :id LIMIT 1";
$stmt = $pdo->prepare($email_query);
$stmt->bindParam(':id', $param_elid, PDO::PARAM_INT);
$stmt->execute();
$email_result = $stmt->fetchColumn();
$hashed_email = calc_hash(mb_strtolower(str_replace(" ", "", $email_result), 'UTF-8'));

// Getting result from FraudHosting
list($report_value, $report_count, $report_reliability, $report_code) = query_fraudhosting($hashed_email, $hashed_ip);

if (empty($report_code)) {
    // Return page redirect to the help page
    $xml = simplexml_load_string($default_xml_string);
    $ok = $xml->addChild('ok', "https://google.com");
    $ok->addAttribute('type', 'url');
    echo $xml->asXML();
    die("Could not get report, redirect to the help page");
}

// Set vars for manual and auto mode checks
if ($check_type == 'manual') {
    $checkCount = isset($moduleConfig['ManualCheckCount']) ? $moduleConfig['ManualCheckCount'] : 1;
    $checkValue = isset($moduleConfig['ManualCheckValue']) ? $moduleConfig['ManualCheckValue'] : 10;
    $checkReliability = isset($moduleConfig['ManualCheckReliability']) ? number_format($moduleConfig['ManualCheckReliability'], 2) : number_format(3.00, 2);
    $groups = $moduleConfig['ManualCheckGroups'];
} else if ($check_type == 'auto') {
    $checkCount = isset($moduleConfig['AutoCheckCount']) ? $moduleConfig['AutoCheckCount'] : 1;
    $checkValue = isset($moduleConfig['AutoCheckValue']) ? $moduleConfig['AutoCheckValue'] : 10;
    $checkReliability = isset($moduleConfig['AutoCheckReliability']) ? number_format($moduleConfig['AutoCheckReliability'], 2) : number_format(3.00, 2);
    $groups = $moduleConfig['AutoCheckGroups'];
}

// Set vars for Telegram notifications
$TelegramBotToken = trim($moduleConfig['TelegramBotToken']);
$TelegramMessage = trim($moduleConfig['TelegramMessage']);
$TelegramChatId = trim($moduleConfig['TelegramChatId']);
$TelegramNotify = trim($moduleConfig['TelegramNotify']);

if (!empty($TelegramNotify) && !empty($TelegramBotToken) && !empty($TelegramChatId) && !empty($TelegramMessage)) {
    if ($check_type == 'manual' && (strpos($TelegramNotify, 'mc') !== false)) {
        $TelegramNeedNotify = true;
    } else if ($check_type == 'auto' && (strpos($TelegramNotify, 'ac') !== false)) {
        $TelegramNeedNotify = true;
    }
}

// Set vars for custom webhook 
$CustomNotify = trim($moduleConfig['CustomNotify']);
$CustomUrl = trim($moduleConfig['CustomUrl']);
$CustomMethod = trim($moduleConfig['CustomMethod']);
$CustomHeaders = trim($moduleConfig['CustomHeaders']);
$CustomData = trim($moduleConfig['CustomData']);

if (!empty($CustomNotify) && !empty($CustomUrl) && !empty($CustomMethod) && !empty($CustomHeaders)) {
    if (($check_type == 'manual') && (strpos($CustomNotify, 'mc') !== false)) {
        $WebhookNeedNotify = true;
    } else if (($check_type == 'auto') && (strpos($CustomNotify, 'ac') !== false)) {
        $WebhookNeedNotify = true;
    }
}

// Check if report exceed threshold values
Info("Threshold values: (" . $checkValue . "-" . $checkCount . "-" . $checkReliability . ")");
if (($report_value >= $checkValue) && ($report_count >= $checkCount) && ($report_reliability >= $checkReliability)) {
    Info("Client is BAD");

    // Add client to the specified groups
    $group_ids = explode(',', $groups);
    if (!empty(trim($groups))) {
        foreach ($group_ids as $group_id) {
            addClientToGroup($group_id, $param_elid);
        }
    }

    // Send notifications to the Telegram
    $chat_ids = explode(',', $TelegramChatId);
    if ($TelegramNeedNotify === true) {
        foreach ($chat_ids as $chat_id) {
            send_telegram_message($TelegramBotToken,$chat_id,$TelegramMessage);
        }
    }

    //$url = "https://example.com/api";
    //$method = "POST";
    //$headers = ["Custom-Header: value"];
    //$body = ["key1" => "value1", "key2" => "value2"];

    // Send custom webhook
    if ($WebhookNeedNotify === true) {
        send_webhook($CustomUrl, $CustomMethod, $CustomHeaders, $CustomData);
    }

    // Return page redirect to the report result
    $xml = simplexml_load_string($default_xml_string);
    $ok = $xml->addChild('ok', "https://fraud.hosting/api/?showreport={$report_code}");
    $ok->addAttribute('type', 'url');
    echo $xml->asXML();
} else {
    Info("Client is GOOD");
}
?>