#!/bin/bash
fh_version_value='1.0'

# Define config files
config_path='etc/fh_module.conf'

# Define lang to display messages
if echo "${HTTP_COOKIE}" | egrep -q 'mgrlang5=orion:ru|mgrlang5=dragon:ru'; then lang='ru'; else lang='en'; fi

function httpGet() {
  if command -v curl >/dev/null 2>&1; then
    curl --silent --connect-timeout 3 --max-time 5 -H "User-Agent: FraudHosting billmgr module v$fh_version_value" "$1"
  elif command -v wget >/dev/null 2>&1; then
    wget -qO- --timeout=5 --tries=1 --header="User-Agent: FraudHosting billmgr module v$fh_version_value" "$1"
  else
    return 1
  fi
}
function checkPluginUpdate() {
  URL="https://fraud.hosting/downloads/billmgr/latest.version"
  response=$(httpGet "$URL")
  if ! echo "$response" | egrep -q '^[0-9]+\.[0-9]+$'; then
    new_version_available="unknown"
  elif [[ $fh_version_value != $response ]]; then
    new_version_available="$response"
  else
    new_version_available="no"
  fi
}
function getFromConfig() {
  grep -w "$1" "$config_path" | awk '{print $2}'
}
function parseConfig() {
  config=$(cat $config_path)

  # Fraudhosting token settings
  fh_token_value=$(getFromConfig 'FraudhostingToken') 

  # Manual check settings
  mc_report_count_value=$(getFromConfig 'ManualCheckCount')
  mc_report_value_value=$(getFromConfig 'ManualCheckValue')
  mc_report_reliability_value=$(getFromConfig 'ManualCheckReliability')
  mc_groups_value=$(getFromConfig 'ManualCheckGroups')

  # Auto check settings
  ac_check_value=$(getFromConfig 'AutoCheck') 
  ac_report_count_value=$(getFromConfig 'AutoCheckCount')  
  ac_report_value_value=$(getFromConfig 'AutoCheckValue' )
  ac_report_reliability_value=$(getFromConfig 'AutoCheckReliability')
  ac_groups_value=$(getFromConfig 'AutoCheckGroups')

  # Telegram Bot API settings
  tg_notify_value=$(getFromConfig 'TelegramNotify')
  tg_token_value=$(getFromConfig 'TelegramBotToken')
  tg_chat_value=$(getFromConfig 'TelegramChatId')
  tg_message_value=$(escape_xml "$(getFromConfig 'TelegramMessage' | base64 -d)")
  tg_nt_types=""; IFS=',' read -ra types <<< "$tg_notify_value"; for tg_notify in "${types[@]}"; do tg_nt_types+="<tg_notify>$tg_notify</tg_notify>"; done

  # Slack Bot API settings
  sk_notify_value=$(getFromConfig 'SlackNotify')
  sk_token_value=$(getFromConfig 'SlackBotToken')
  sk_chat_value=$(getFromConfig 'SlackChatId')
  sk_message_value=$(escape_xml "$(getFromConfig 'SlackMessage' | base64 -d)")
  sk_nt_types=""; IFS=',' read -ra types <<< "$sk_notify_value"; for sk_notify in "${types[@]}"; do sk_nt_types+="<sk_notify>$sk_notify</sk_notify>"; done

  # Mattermost Bot API settings
  mm_notify_value=$(getFromConfig 'MattermostNotify')
  mm_token_value=$(getFromConfig 'MattermostBotToken')
  mm_chat_value=$(getFromConfig 'MattermostChatId')
  mm_message_value=$(escape_xml "$(getFromConfig 'MattermostMessage' | base64 -d)")
  mm_nt_types=""; IFS=',' read -ra types <<< "$mm_notify_value"; for mm_notify in "${types[@]}"; do mm_nt_types+="<mm_notify>$mm_notify</mm_notify>"; done

  # Discord Bot API settings
  ds_notify_value=$(getFromConfig 'DiscordNotify')
  ds_token_value=$(getFromConfig 'DiscordBotToken')
  ds_chat_value=$(getFromConfig 'DiscordChatId')
  ds_message_value=$(escape_xml "$(getFromConfig 'DiscordMessage' | base64 -d)")
  ds_nt_types=""; IFS=',' read -ra types <<< "$ds_notify_value"; for ds_notify in "${types[@]}"; do ds_nt_types+="<ds_notify>$ds_notify</ds_notify>"; done

  # Custom webhook settings
  cw_notify_value=$(getFromConfig 'CustomNotify')
  cw_url_value=$(escape_xml "$(getFromConfig 'CustomUrl')")
  cw_method_value=$(getFromConfig 'CustomMethod')
  cw_headers_value=$(escape_xml "$(getFromConfig 'CustomHeaders' | base64 -d)")
  cw_data_value=$(escape_xml "$(getFromConfig 'CustomData' | base64 -d)")
  cw_nt_types=""; IFS=',' read -ra types <<< "$cw_notify_value"; for cw_notify in "${types[@]}"; do cw_nt_types+="<cw_notify>$cw_notify</cw_notify>"; done
}
function putToConfig() {
  sed -i "/\b$1\b/d" "$config_path"
  if [[ $2 != '' ]]; then
    echo "$1 $2" >> $config_path
  fi
}
function saveConfig() {
  # Fraudhosting token settings
  putToConfig "FraudhostingToken" "${PARAM_fh_token}"

  # Manual check settings
  putToConfig "ManualCheckCount" "${PARAM_mc_report_count}"
  putToConfig "ManualCheckValue" "${PARAM_mc_report_value}"
  putToConfig "ManualCheckReliability" "${PARAM_mc_report_reliability}"
  putToConfig "ManualCheckGroups" "${PARAM_mc_groups}"

  # Auto check settings
  putToConfig "AutoCheck" "${PARAM_ac_check:-off}"
  putToConfig "AutoCheckCount" "${PARAM_ac_report_count}"
  putToConfig "AutoCheckValue" "${PARAM_ac_report_value}"
  putToConfig "AutoCheckReliability" "${PARAM_ac_report_reliability}"
  putToConfig "AutoCheckGroups" "${PARAM_ac_groups}"

  # Telegram Bot API settings
  putToConfig "TelegramNotify" "${PARAM_tg_notify}"
  putToConfig "TelegramBotToken" "${PARAM_tg_token}"
  putToConfig "TelegramChatId" "${PARAM_tg_chat}"
  putToConfig "TelegramMessage" $(echo -n "${PARAM_tg_message}" | base64 -w 0)

  # Slack Bot API settings
  putToConfig "SlackNotify" "${PARAM_sk_notify}"
  putToConfig "SlackBotToken" "${PARAM_sk_token}"
  putToConfig "SlackChatId" "${PARAM_sk_chat}"
  putToConfig "SlackMessage" $(echo -n "${PARAM_sk_message}" | base64 -w 0)

  # Mattermost Bot API settings
  putToConfig "MattermostNotify" "${PARAM_mm_notify}"
  putToConfig "MattermostBotToken" "${PARAM_mm_token}"
  putToConfig "MattermostChatId" "${PARAM_mm_chat}"
  putToConfig "MattermostMessage" $(echo -n "${PARAM_mm_message}" | base64 -w 0)

  # Discord Bot API settings
  putToConfig "DiscordNotify" "${PARAM_ds_notify}"
  putToConfig "DiscordBotToken" "${PARAM_ds_token}"
  putToConfig "DiscordChatId" "${PARAM_ds_chat}"
  putToConfig "DiscordMessage" $(echo -n "${PARAM_ds_message}" | base64 -w 0)

  # Custom webhook settings
  putToConfig "CustomNotify" "${PARAM_cw_notify}"
  putToConfig "CustomUrl" "${PARAM_cw_url}"
  putToConfig "CustomMethod" "${PARAM_cw_method}"
  putToConfig "CustomHeaders" $(echo -n "${PARAM_cw_headers}" | base64 -w 0)
  putToConfig "CustomData" $(echo -n "${PARAM_cw_data}" | base64 -w 0)
}
function escape_xml() {
  local input="$1"
  input="${input//&/&#38;}"
  input="${input//</&#60;}"
  input="${input//>/&#62;}"
  input="${input//\'/&#39;}"
  input="${input//\"/&#34;}"
  echo "$input"
}

# If form sent, save config file, then exit
if [[ "${PARAM_func}" == "fraudhosting.settings" && "${PARAM_clicked_button}" == "save" ]]; then
  saveConfig
  echo '<?xml version="1.0" encoding="UTF-8"?><doc func="fraudhosting.settings"></doc>'
  exit 0
fi

# If form not sent, parese config file, then display form
parseConfig

# Check if plugin update available
checkPluginUpdate

# Set placeholders values
placeholder_report_value='10'
placeholder_report_count='1'
placeholder_report_reliability='3.00'

placeholder_groups=$(escape_xml '123,456,789')

placeholder_tg_token=$(escape_xml '1234567890:AABBCCDDEEFF..XX')
placeholder_tg_chat=$(escape_xml '123,654,-987')
placeholder_tg_message=$(escape_xml 'Check type: {check_type}
Reg IP: {reg_ip}
Reg email: {reg_email}
Report value: {report_value}
Report count: {report_count}
Report reliability: {report_reliability}
<a href="https://fraud.hosting/api/?showreport={report_code}">View report</a>')

placeholder_cw_url=$(escape_xml "https://domain.tld/path/to/script")
placeholder_cw_headers=$(escape_xml "Content-type: application/json
Authorization: Bearer <token>
User-Agent: FraudHosting module v$fh_version_value
Cookie: session=<value>")
placeholder_cw_data=$(escape_xml '{
  "check_type": "{check_type}",
  "reg_ip": "{reg_ip}",
  "reg_email": "{reg_email}",
  "report_value": "{report_value}",
  "report_count": "{report_count}",
  "report_reliability": "{report_reliability}",
  "report_code": "{report_code}",
  "report_url": "https://fraud.hosting/api/?showreport={report_code}"
}')

if [[ $lang == 'ru' ]]; then
  fh_token_settings='Основные настройки'
  fh_token='FraudHosting API ключ'
  hint_fh_token='Указанный API ключ будет использоваться для всех запросов к FraudHosting, ключ можно получить в личном кабинете FraudHosting'
  fh_version='Версия плагина интеграции'
  if [[ $new_version_available == 'unknown' ]]; then
    fh_update='<![CDATA[<font color="red">Не удалось получить последнюю доступную версию плагина, проверьте вручную по <a href="https://www.google.com" target="_blank">ссылке</a></font>]]>'
  elif [[ $new_version_available == 'no' ]]; then
    fh_update='<![CDATA[<font color="green">У вас установлена последняя доступная версия плагина</font>]]>'
  else
    fh_update='<![CDATA[<font color="yellow">Доступно обновление плагина v'$new_version_available', загрузите обновление по <a href="https://www.google.com" target="_blank">ссылке</a></font>]]>'
  fi

  fh_mc_settings='Режим ручной проверки'
  mc_info='<![CDATA[При выполнении ручной проверки клиента, будет сформирован запрос в FraudHosting, запрос будет содержать хешированные значения <font color="yellow">IP адреса регистрации</font> и <font color="yellow">Email адреса</font> клиента.<br/>Плохим результатом считается если все значения пройдут порог срабатывания.<br/>В случае плохой проверки клиент добавится в указанные ID групп, а также опционально может быть отправлено уведомление или Webhook.]]>'
  mc_report_value='Сумма баллов нарушений >='
  mc_report_count='Количество репортов >='
  mc_report_reliability='Надежность отчета >='
  mc_groups='Добавить клиента в группу ID'
  hint_mc_groups='При срабатывании условий, клиент будет автоматически добавлен в группы с указанными ID. Можно указать несколько ID через зяпятую'
  hint_mc_report_value='Сумма баллов нарушений со всех репортов отправленных другими участниками'
  hint_mc_report_count='Количество репортов отправленных другими участниками'
  hint_mc_report_reliability='Надежность отчета, значение от 1.00 до 10.00. Значение зависит от кол-ва репортов, баллов нарушений и серьезности нарушений'

  fh_ac_settings='Режим автоматической проверки'
  ac_info='<![CDATA[Сразу после регистрации нового клиента, будет сформирован запрос в FraudHosting, запрос будет содержать хешированные значения <font color="yellow">IP адреса регистрации</font> и <font color="yellow">Email адреса</font> клиента.<br/>Плохим результатом считается если все значения пройдут порог срабатывания.<br/>В случае плохой проверки клиент добавится в указанные ID групп, а также опционально может быть отправлено уведомление или Webhook.]]>'
  ac_check='Автоматическая проверка клиентов'
  hint_ac_check='Когда включено, автоматическая проверка клиентов будет запускаться после регистрации'
  ac_report_value='Сумма баллов нарушений >='
  ac_report_count='Количество репортов >='
  ac_report_reliability='Надежность отчета >='
  ac_groups='Добавить клиента в группу ID'
  hint_ac_groups='При срабатывании условий, клиент будет автоматически добавлен в группы с указанными ID. Можно указать несколько ID через зяпятую'
  hint_ac_report_value='Сумма баллов нарушений со всех репортов отправленных другими участниками'
  hint_ac_report_count='Количество репортов отправленных другими участниками'
  hint_ac_report_reliability='Надежность отчета, значение от 1.00 до 10.00. Значение зависит от кол-ва репортов, баллов нарушений и серьезности нарушений'

  fh_tg_settings='Уведомления в Telegram'
  tg_notify='Использовать для проверок'
  hint_tg_notify='Выберите при каких проверках должно отправляться уведомление. Уведомление будет отправлено только при плохой проверке.'
  tg_token='Telegram Bot API ключ'
  hint_tg_token='Указанный Telegram Bot API ключ будет использоваться для всех запросов к Telegram, ключ можно получить у бота @botfather'
  tg_chat='Telegram Chat ID'
  hint_tg_chat='Указанный Telegram Chat ID будет использоваться для отправки сообщений. можно указать несколько значений через зяпятую.'
  tg_message='Сообщение для отправки'
  hint_tg_message='Содержимое сообщения которое будет отправлено. Формат сообщения должен быть HTML, можно использовать переменные: {reg_ip}, {reg_email}, {report_value}, {report_count}, {report_reliability}, {report_code}, {check_type}'

  fh_cw_settings='Пользовательский Webhook'
  cw_notify='Использовать для проверок'
  hint_cw_notify='Выберите при каких проверках должно отправляться уведомление. Уведомление будет отправлено только при плохой проверке.'
  cw_url='URL'
  hint_cw_url='На указанный Webhook URL будет выполнен cURL запрос'
  cw_headers='Заголовки'
  hint_cw_headers='Заголовки которые будут установлены для cURL запроса'
  cw_method='Метод'
  hint_cw_method='Метод запроса. GET / POST'
  cw_data='Данные'
  hint_cw_data='Данные которые будут отправлены в запросе'

  nt_mc='ручные'
  nt_ac="автоматические"

  hint_reload='Обновить форму'
  hint_copylink='Скопировать ссылку на форму'
  hint_takefavorite='Добавить в избранное меню'
  hint_takeunfavorite='Убрать из избранного меню'
  hint_pin='Закрепить вкладку'
  hint_unpin='Открепить вкладку'
  msg_save='Сохранить'
  msg_cancel='Отмена'
  title='Настройки модуля FraudHosting'
else
  fh_token_settings='Main settings'
  fh_token='FraudHosting API token'
  hint_fh_token='Specified API token will be using for all requests to FraudHosting, you can get API token from client area FraudHosting'
  fh_version='Integration plugin version'
  if [[ $new_version_available == 'unknown' ]]; then
    fh_update='<![CDATA[<font color="red">Failed to get available plugin version from the server, check if new version is available manually by <a href="https://www.google.com" target="_blank">following link</a></font>]]>'
  elif [[ $new_version_available == 'no' ]]; then
    fh_update='<![CDATA[<font color="green">You have latest available version installed</font>]]>'
  else
    fh_update='<![CDATA[<font color="yellow">Available new version of intergration plugin v'$new_version_available', please download new version by <a href="https://www.google.com" target="_blank">following link</a></font>]]>'
  fi

  fh_mc_settings='Manual check mode'
  mc_info='<![CDATA[When performing manual client verification, a request will be generated to FraudHosting, the request will contain hashed values of the <font color="yellow">registration IP address</font> and <font color="yellow">Email address</font> of the client.<br/>A bad result is considered if all values pass the threshold.<br/>In case of a bad check, the client will be added to the specified groups, and optionally a notification or Webhook can be sent.]]>'
  mc_report_value='Report Value >='
  mc_report_count='Report Count >='
  mc_report_reliability='Report Reliability >='
  mc_groups='Add client to the groups ID'
  hint_ac_groups='When the conditions are triggered, the client will be automatically added to the groups with the specified IDs. You can specify several IDs separated by commas'
  hint_mc_report_value='Total report points from all reports sent by other members'
  hint_mc_report_count='Number of reports sent by other members'
  hint_mc_report_reliability='Report reliability, values from 1.00 to 10.00. The value depends on the number of reports, report points and severity of violations'

  fh_ac_settings='Automatic check mode'
  ac_info='<![CDATA[After client registration, a request will be generated to FraudHosting, the request will contain hashed values of the <font color="yellow">registration IP address</font> and <font color="yellow">Email address</font> of the client.<br/>A bad result is considered if all values pass the threshold.<br/>In case of a bad check, the client will be added to the specified groups, and optionally a notification or Webhook can be sent.]]>'
  ac_check='Auto check clients'
  hint_ac_check='When enabled, automatically check client after registration'
  ac_report_value='Report Value >='
  ac_report_count='Report Count >='
  ac_report_reliability='Report Reliability >='
  ac_groups='Add client to the groups ID'
  hint_ac_groups='When the conditions are triggered, the client will be automatically added to the groups with the specified IDs. You can specify several IDs separated by commas'
  hint_ac_report_value='Total report points from all reports sent by other members'
  hint_ac_report_count='Number of reports sent by other members'
  hint_ac_report_reliability='Report reliability, value from 1.00 to 10.00. The value depends on the number of reports, report points and severity of violations'

  fh_tg_settings='Notifications to Telegram'
  tg_notify='Notifications triggers'
  hint_tg_notify='Select triggers to send notifications. The notification will only be sent if the check is bad.'
  tg_token='Telegram Bot API token'
  hint_tg_token='Specified Telegram Bot API token will be using for all requests to Telegram, you can get API token from @botfather bot'
  tg_chat='Telegram Chat ID'
  hint_tg_chat='Specified Telegram Chat ID will be using for sending messages. Multiple values comma separated available.'
  tg_message='Message to send'
  hint_tg_message='Message content, should be in the HTML format. Available shortcuts: {reg_ip}, {reg_email}, {report_value}, {report_count}, {report_reliability}, {report_code}, {check_type}'

  fh_cw_settings='Сustom Webhook'
  cw_notify='Notifications triggers'
  hint_cw_notify='Select triggers to send notifications. The notification will only be sent if the check is bad.'
  cw_url='URL'
  hint_cw_url='Specified URL will be using for sending webhooks'
  cw_headers='Headers'
  hint_cw_headers='Webhook headers will be using while send requests'
  cw_method='Method'
  hint_cw_method='Webhook request method. GET / POST'
  cw_data='Data'
  hint_cw_data='Webhook data that will be send using Webhook'

  nt_mc='manual'
  nt_ac="auto"

  hint_reload='Reload the form'
  hint_copylink='Copy the link to the form'
  hint_addfavorite='Add to favorites menu'
  hint_removefavorite='Remove from favorites menu'
  hint_pin='Pin the tab'
  hint_unpin='Unpin the tab'
  msg_save='Save'
  msg_cancel='Cancel'
  title='FraudHosting module settings'
fi

# Return form to the billing
cat <<EOF
<?xml version="1.0" encoding="UTF-8"?>
<doc func="fraudhosting.settings">
  <metadata name="fraudhosting.settings" type="form" mgr="billmgr">
    <form>

    <page name="fh_token_settings">
      <field name="fh_token">
        <input type="password" name="fh_token" readonly="no" required='yes'/>
      </field>
      <field name="fh_version">
        <input type="text" name="fh_version" readonly="yes"/>
      </field>
      <field fullwidth="yes" name="fh_update_empty">
        <textdata name="fh_update" type="msg"/>
      </field>
    </page>

    <page name="fh_mc_settings">
      <field fullwidth="yes" name="mc_info_empty">
        <textdata name="mc_info" type="msg"/>
      </field>

      <field name="mc_report_value">
        <input type="text" name="mc_report_value" readonly="no"/>
      </field>

      <field name="mc_report_count">
        <input type="text" name="mc_report_count" readonly="no"/>
      </field>

      <field name="mc_report_reliability">
        <input type="text" name="mc_report_reliability" readonly="no"/>
      </field>

      <field name="mc_groups">
        <input type="text" name="mc_groups" readonly="no"/>
      </field>
    </page>

    <page name="fh_ac_settings">
      <field fullwidth="yes" name="ac_info_empty">
        <textdata name="ac_info" type="msg"/>
      </field>

      <field name="ac_check">
        <input name="ac_check" type="checkbox"/>
      </field>

      <field name="ac_report_value">
        <input type="text" name="ac_report_value" readonly="no"/>
      </field>

      <field name="ac_report_count">
        <input type="text" name="ac_report_count" readonly="no"/>
      </field>

      <field name="ac_report_reliability">
        <input type="text" name="ac_report_reliability" readonly="no"/>
      </field>

      <field name="ac_groups">
        <input type="text" name="ac_groups" readonly="no"/>
      </field>
    </page>

    <page name="fh_tg_settings">
      <field name="tg_notify">
        <select name="tg_notify" type="multiple"/>
      </field>

      <field name="tg_token">
        <input type="password" name="tg_token" readonly="no"/>
      </field>

      <field name="tg_chat">
        <input type="text" name="tg_chat" readonly="no"/>
      </field>

      <field name="tg_message" type="textarea">
        <textarea name="tg_message" rows="10" wrap="off" fullwidth="no" readonly="no"/>
      </field>
    </page>

    <page name="fh_cw_settings">
      <field name="cw_notify">
        <select name="cw_notify" type="multiple"/>
      </field>

      <field name="cw_url">
        <input type="text" name="cw_url" readonly="no"/>
      </field>

      <field name="cw_method">
        <select name="cw_method" type="radio">
          <if value="get" hide="cw_data"/>
        </select>
      </field>

      <field name="cw_headers" type="textarea">
        <textarea name="cw_headers" rows="6" wrap="off" fullwidth="no" readonly="no"/>
      </field>

      <field name="cw_data" type="textarea">
        <textarea name="cw_data" rows="10" wrap="off" fullwidth="no" readonly="no"/>
      </field>
    </page>

      <buttons>
        <button name="save" type="ok"/>
        <button name="cancel" type="cancel"/>
      </buttons>
    </form>
  </metadata>

  <slist name="tg_notify">
    <val key="mc">$nt_mc</val>
    <val key="ac">$nt_ac</val>
  </slist>

  <slist name="cw_method">
    <val key="get">GET</val>
    <val key="post">POST</val>
  </slist>

  <slist name="cw_notify">
    <val key="mc">$nt_mc</val>
    <val key="ac">$nt_ac</val>
  </slist>

  <messages name="fraudhosting.settings">
    <msg name="fh_token_settings">$fh_token_settings</msg>
    <msg name="fh_token">$fh_token</msg>
    <msg name="hint_fh_token">$hint_fh_token</msg>

    <msg name="fh_version">$fh_version</msg>
    <msg name="fh_update">$fh_update</msg>

    <msg name="fh_mc_settings">$fh_mc_settings</msg>
    <msg name="mc_info">$mc_info</msg>
    <msg name="mc_report_value">$mc_report_value</msg>
    <msg name="hint_mc_report_value">$hint_mc_report_value</msg>
    <msg name="placeholder_mc_report_value">$placeholder_report_value</msg>
    <msg name="mc_report_count">$mc_report_count</msg>
    <msg name="hint_mc_report_count">$hint_mc_report_count</msg>
    <msg name="placeholder_mc_report_count">$placeholder_report_count</msg>
    <msg name="mc_report_reliability">$mc_report_reliability</msg>
    <msg name="hint_mc_report_reliability">$hint_mc_report_reliability</msg>
    <msg name="placeholder_mc_report_reliability">$placeholder_report_reliability</msg>
    <msg name="mc_groups">$mc_groups</msg>
    <msg name="hint_mc_groups">$hint_mc_groups</msg>
    <msg name="placeholder_mc_groups">$placeholder_groups</msg>

    <msg name="fh_ac_settings">$fh_ac_settings</msg>
    <msg name="ac_info">$ac_info</msg>
    <msg name="ac_check">$ac_check</msg>
    <msg name="hint_ac_check">$hint_ac_check</msg>
    <msg name="ac_report_value">$ac_report_value</msg>
    <msg name="hint_ac_report_value">$hint_ac_report_value</msg>
    <msg name="placeholder_ac_report_value">$placeholder_report_value</msg>
    <msg name="ac_report_count">$ac_report_count</msg>
    <msg name="hint_ac_report_count">$hint_ac_report_count</msg>
    <msg name="placeholder_ac_report_count">$placeholder_report_count</msg>
    <msg name="ac_report_reliability">$ac_report_reliability</msg>
    <msg name="hint_ac_report_reliability">$hint_ac_report_reliability</msg>
    <msg name="placeholder_ac_report_reliability">$placeholder_report_reliability</msg>
    <msg name="ac_groups">$ac_groups</msg>
    <msg name="hint_ac_groups">$hint_ac_groups</msg>
    <msg name="placeholder_ac_groups">$placeholder_groups</msg>

    <msg name="fh_tg_settings">$fh_tg_settings</msg>
    <msg name="tg_notify">$tg_notify</msg>
    <msg name="hint_tg_notify">$hint_tg_notify</msg>
    <msg name="tg_token">$tg_token</msg>
    <msg name="hint_tg_token">$hint_tg_token</msg>
    <msg name="placeholder_tg_token">$placeholder_tg_token</msg>
    <msg name="tg_chat">$tg_chat</msg>
    <msg name="hint_tg_chat">$hint_tg_chat</msg>
    <msg name="placeholder_tg_chat">$placeholder_tg_chat</msg>
    <msg name="tg_message">$tg_message</msg>
    <msg name="hint_tg_message">$hint_tg_message</msg>
    <msg name="placeholder_tg_message">$placeholder_tg_message</msg>

    <msg name="fh_cw_settings">$fh_cw_settings</msg>
    <msg name="cw_notify">$cw_notify</msg>
    <msg name="hint_cw_notify">$hint_cw_notify</msg>
    <msg name="cw_url">$cw_url</msg>
    <msg name="hint_cw_url">$hint_cw_url</msg>
    <msg name="placeholder_cw_url">$placeholder_cw_url</msg>
    <msg name="cw_headers">$cw_headers</msg>
    <msg name="hint_cw_headers">$hint_cw_headers</msg>
    <msg name="placeholder_cw_headers">$placeholder_cw_headers</msg>
    <msg name="cw_method">$cw_method</msg>
    <msg name="hint_cw_method">$hint_cw_method</msg>
    <msg name="cw_data">$cw_data</msg>
    <msg name="hint_cw_data">$hint_cw_data</msg>
    <msg name="placeholder_cw_data">$placeholder_cw_data</msg>

    <msg name="hint_reload">$hint_reload</msg>
    <msg name="hint_copylink">$hint_copylink</msg>
    <msg name="hint_takefavorite">$hint_takefavorite</msg>
    <msg name="hint_takeunfavorite">$hint_takeunfavorite</msg>
    <msg name="hint_pin">$hint_pin</msg>
    <msg name="hint_unpin">$hint_unpin</msg>
    <msg name="msg_save">$msg_save</msg>
    <msg name="msg_cancel">$msg_cancel</msg>
    <msg name="title">$title</msg>
  </messages>

  <fh_token>$fh_token_value</fh_token>
  <fh_version>$fh_version_value</fh_version>

  <mc_check>$mc_check_value</mc_check>
  <mc_report_count>$mc_report_count_value</mc_report_count>
  <mc_report_value>$mc_report_value_value</mc_report_value>
  <mc_report_reliability>$mc_report_reliability_value</mc_report_reliability>
  <mc_groups>$mc_groups_value</mc_groups>

  <ac_check>$ac_check_value</ac_check>
  <ac_report_count>$ac_report_count_value</ac_report_count>
  <ac_report_value>$ac_report_value_value</ac_report_value>
  <ac_report_reliability>$ac_report_reliability_value</ac_report_reliability>
  <ac_groups>$ac_groups_value</ac_groups>

  $tg_nt_types
  <tg_token>$tg_token_value</tg_token>
  <tg_chat>$tg_chat_value</tg_chat>
  <tg_message>$tg_message_value</tg_message>

  $sk_nt_types
  <sk_token>$sk_token_value</sk_token>
  <sk_chat>$sk_chat_value</sk_chat>
  <sk_message>$sk_message_value</sk_message>

  $mm_nt_types
  <mm_token>$mm_token_value</mm_token>
  <mm_chat>$mm_chat_value</mm_chat>
  <mm_message>$mm_message_value</mm_message>

  $ds_nt_types
  <ds_token>$ds_token_value</ds_token>
  <ds_chat>$ds_chat_value</ds_chat>
  <ds_message>$ds_message_value</ds_message>

  $cw_nt_types
  <cw_url>$cw_url_value</cw_url>
  <cw_method>$cw_method_value</cw_method>
  <cw_headers>$cw_headers_value</cw_headers>
  <cw_data>$cw_data_value</cw_data>
</doc>
EOF