<?php
date_default_timezone_set('UTC');
$log_file = fopen('/usr/local/mgr5/var/'. __MODULE__ .'.log', 'a');
$default_xml_string = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<doc/>\n";
$pid = getmypid();

function Debug($str) {
    global $log_file;
    global $pid;
    fwrite($log_file, date("Y-m-d H:i:s") ." [". $pid ."] ". __MODULE__ ." \033[1;33mDEBUG ". $str ."\033[0m\n");
}

function Error($str) {
    global $log_file;
    global $pid;
    fwrite($log_file, date("Y-m-d H:i:s") ." [". $pid ."] ". __MODULE__ ." \033[1;31mERROR ". $str ."\033[0m\n");
}

function Info($str) {
    global $log_file;
    global $pid;
    fwrite($log_file, date("Y-m-d H:i:s") ." [". $pid ."] ". __MODULE__ ." \033[1;32mINFO ". $str ."\033[0m\n");
}

function parseConfig($configFile) {
    if (!file_exists($configFile)) { throw new Exception("Config not found: " . $configFile); }
    $config = [];
    $lines = file($configFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    foreach ($lines as $line) {
        if (strpos(trim($line), '#') === 0) { continue; }
        list($key, $value) = explode(' ', $line, 2);
        $key = trim($key);
        $value = trim($value);
        $config[$key] = $value;
    }
    return $config;
}

function calc_hash($value) {
    $value = strtolower(trim($value));
    Info("Create hash for: " . $value);
    for ($i = 0; $i < 32000; $i++) {
        $value = sha1("fraudhosting-" . $value);
    }
    Info("Done, hash is: " . $value);
    return $value;
}

function query_fraudhosting($hash_email, $hash_regip) {
    global $moduleConfig;
    $url = "https://api.fraud.hosting/api/";
    $fields = [
        '_action' => 'query',
        '_api' => $moduleConfig['FraudhostingToken'],
        'email' => $hash_email,
        'ip' => $hash_regip
    ];
    $fields_string = http_build_query($fields);
    $ch = curl_init();
    curl_setopt($ch,CURLOPT_URL, $url);
    curl_setopt($ch,CURLOPT_POST, true);
    curl_setopt($ch,CURLOPT_POSTFIELDS, $fields_string);
    curl_setopt($ch,CURLOPT_RETURNTRANSFER, true); 
    $response = curl_exec($ch);


    $start_tag = "<report>";
    $end_tag = "</report>";
    if (strpos($response, $start_tag) !== false && strpos($response, $end_tag) !== false) {
        $start_index = strpos($response, $start_tag) + strlen($start_tag);
        $end_index = strpos($response, $end_tag);
        $report_content = substr($response, $start_index, $end_index - $start_index);
        $report_parts = explode('-', $report_content);
        if (count($report_parts) === 4) {
            $report_value = (int)$report_parts[0];
            $report_count = (int)$report_parts[1];
            $report_reliability = number_format($report_parts[2],2);
            $report_code = $report_parts[3];
            Info("FraudHosting API Response: " . $response);
            return [$report_value, $report_count, $report_reliability, $report_code];
        }
    } else {
        Error("Invalid response from FraudHosting API: " . $response);
        return [null, null, null, null];
    }
}

function report_fraudhosting($hash_email, $hash_regip, $text, $type, $value) {
    global $moduleConfig;
    $url = "https://api.fraud.hosting/api/";
    $fields = [
        '_action' => 'report',
        '_api' => $moduleConfig['FraudhostingToken'],
     	'_text' => $text,
        '_type' => $type,
        '_value' => $value,
        'email' => $hash_email,
        'ip' => $hash_regip
    ];
    $ch = curl_init();
    curl_setopt($ch,CURLOPT_URL, $url);
    curl_setopt($ch,CURLOPT_POST, true);
	curl_setopt($ch,CURLOPT_POSTFIELDS,$fields);
	curl_setopt($ch,CURLOPT_RETURNTRANSFER,1);
    $response = curl_exec($ch);

    if (strpos($response, "OK") !== false) {
        $report_parts = explode(':', $response);
        if (count($report_parts) === 2) {
            $report_code = $report_parts[1];
            Info("FraudHosting API Response: " . $response);
            return $report_code;
        }
    } else {
        Error("Invalid response from FraudHosting API: " . $response);
        return null;
    }
}

function shortcut_replace($message_text) {
    global $check_type, $ip_result, $email_result, $report_value, $report_count, $report_reliability, $report_code;
    $replacements = [
        '{check_type}' => $check_type,
        '{reg_ip}' => $ip_result,
        '{reg_email}' => $email_result,
        '{report_value}' => $report_value,
        '{report_count}' => $report_count,
        '{report_reliability}' => $report_reliability,
        '{report_code}' => $report_code,
    ];
    $message_text = str_replace(array_keys($replacements), array_values($replacements), $message_text);
    return $message_text;
}

function send_telegram_message($bot_token, $chat_id, $message_text) {
    $message_text = shortcut_replace(base64_decode($message_text));
    $url = "https://api.telegram.org/bot{$bot_token}/sendMessage";
    $data = [
        'chat_id' => trim($chat_id),
        'text' => $message_text,
        'parse_mode' => 'HTML'
    ];
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
    $response = curl_exec($ch);
    if (curl_errno($ch)) {
        Error("Telegram send failed to chat ID $chat_id: " . curl_error($ch));
    } else {
        Info("Telegram response to chat ID $chat_id: " . $response);
    }
    curl_close($ch);
    return $response;
}

function send_webhook($url, $method, $headers, $body = null) {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_TIMEOUT, 10);

    $headers = base64_decode($headers);
    $headers_array = explode("\n", trim($headers)); // Разбиваем строку на массив строк по символу новой строки
    $headers_array = array_filter($headers_array, 'trim'); // Удаляем пустые строки
    if (!empty($headers_array)) {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers_array);
    }
    Info("$headers");

    $body = shortcut_replace(base64_decode($body));
    Info("$body");

    Info("$method");
    if ($method === 'post') {
        curl_setopt($ch, CURLOPT_POST, true);
        if (!empty($body)) {
            //json_decode($body);
            //if (json_last_error() === JSON_ERROR_NONE) {
                curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
            //} else { 
            //    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($body));
            //}
        }
    }

    $url = shortcut_replace($url);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    if (curl_errno($ch)) {
        Error("Webhook send failed: " . curl_error($ch));
    } else {
        Info("Webhook response: " . $response);
    }
    curl_close($ch);
    return $response;
}

function connectToDatabase($host, $user, $password, $dbname) {
    $pdo = new PDO("mysql:host={$host};dbname={$dbname};charset=utf8mb4", $user, $password);
    $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
    return $pdo;
}

function addClientToGroup($group_id, $client_id) {
    $group_id = trim($group_id);
    $client_id = trim($client_id);
    shell_exec("sbin/mgrctl -m billmgr account.group.resume elid={$group_id} plid={$client_id} su=admin");
    Info("Add client to group ID $group_id");
}

function escapeForXML($string) {
    return htmlspecialchars($string, ENT_XML1, 'UTF-8');
}

function parseNameAndEmail($inputString) {
    $parts = explode(" (", $inputString);
    if (count($parts) !== 2) {
        return [null, null];
    }
    $name = trim($parts[0]);
    $email = trim(str_replace(")", "", $parts[1]));
    return [$name, $email];
}

function defineLang() {
    global $param_cookie;
    if ((strpos($param_cookie, 'mgrlang5=orion:ru') !== false) || (strpos($param_cookie, 'mgrlang5=dragon:ru') !== false)) {
        return 'ru';
    } else {
        return 'en';
    }
}
?>