#!/usr/bin/env php
<?php
set_include_path(get_include_path() . PATH_SEPARATOR . '/usr/local/mgr5/include/php');
define('__MODULE__', 'fraudhosting');
require_once 'fraudhosting_func.php';

Info("FraudHosting plugin started");

// Define config files
$moduleConfig = parseConfig('etc/fh_module.conf');
$dbConfig = parseConfig('etc/billmgr.conf.d/db.conf');

// Get env params
$param_func = getenv('PARAM_func');
$param_elid = getenv('PARAM_elid');
$param_out = getenv('PARAM_out');
$param_authlevel = getenv('AUTH_LEVEL');
$param_authuser = getenv('AUTH_USER');
$param_cookie = getenv('HTTP_COOKIE');
$param_clicked_button = getenv('PARAM_clicked_button');

$param_type = getenv('PARAM_type');
$param_value = getenv('PARAM_value');
$param_text = getenv('PARAM_text');

$param_regip = getenv('PARAM_regip');
$param_name = getenv('PARAM_name');
$param_email = getenv('PARAM_email');

// Stop, if non matched params
if ($param_authlevel !== '29') { die("Auth level not admin"); }
if ($param_func !== 'fraudhosting.report') { die("Requested function is wrong"); }
if (empty($param_elid)) { die("Client ID (elid) is empty"); }

try {
  $pdo = connectToDatabase($dbConfig['DBHost'], $dbConfig['DBUser'], $dbConfig['DBPassword'], $dbConfig['DBName']);
} catch (PDOException $e) {
  if ($dbConfig['DBHost'] === 'localhost') {
      try {
          $pdo = connectToDatabase('127.0.0.1', $dbConfig['DBUser'], $dbConfig['DBPassword'], $dbConfig['DBName']);
      } catch (PDOException $e) {
          Error("Database connection failed: " . $e->getMessage());
          die("Unable to connect to the database.");
      }
  } else {
      Error("Database connection failed: " . $e->getMessage());
      die("Unable to connect to the database.");
  }
}

// Getting client register IP and create hash
$ip_query = "SELECT registration_ip FROM account WHERE id = :id";
$stmt = $pdo->prepare($ip_query);
$stmt->bindParam(':id', $param_elid, PDO::PARAM_INT);
$stmt->execute();
$regip_value = $stmt->fetchColumn();

// Getting first user email for client and create hash
$email_query = "SELECT email FROM user WHERE account = :id LIMIT 1";
$stmt = $pdo->prepare($email_query);
$stmt->bindParam(':id', $param_elid, PDO::PARAM_INT);
$stmt->execute();
$email_value = $stmt->fetchColumn();

// Getting first user realname for client and create hash
$realname_query = "SELECT realname FROM user WHERE account = :id LIMIT 1";
$stmt = $pdo->prepare($realname_query);
$stmt->bindParam(':id', $param_elid, PDO::PARAM_INT);
$stmt->execute();
$realname_value = $stmt->fetchColumn();

// Define lang to display messages
$lang = defineLang();

# Send report to FraudHosting if form is sent, then exit
if ($param_func === "fraudhosting.report" && $param_clicked_button === "ok" ) {

  $hashed_ip = calc_hash(mb_strtolower(str_replace(" ", "", $regip_value), 'UTF-8'));
  $hashed_email = calc_hash(mb_strtolower(str_replace(" ", "", $email_value), 'UTF-8'));
  $hashed_realname = calc_hash(mb_strtolower(str_replace(" ", "", $realname_value), 'UTF-8'));
  
  switch ($param_type) {
    case too_many_support_tickets:
      $param_type = 'Too many support tickets';
      break;
    case chargeback:
      $param_type = 'Chargeback';
      break;
    case stolen_card:
      $param_type = 'Stolen card';
      break;
    case fraud:
      $param_type = 'Fraud';
      break;
    case early_cancellation:
      $param_type = 'Early cancellation';
      break;
    case dmca:
      $param_type = 'DMCA';
      break;
    case excessive_server_load:
      $param_type = 'Excessive server load';
      break;
    case public_threats:
      $param_type = 'Public threats';
      break;
    case drugs_content:
      $param_type = 'Drugs content';
      break;
    case sexual_exploitation:
      $param_type = 'Sexual exploitation';
      break;
    case malwares:
      $param_type = 'Malwares';
      break;
    case port_scan:
      $param_type = 'Port Scan';
      break;
    case phishing:
      $param_type = 'Phishing';
      break;
    case spam:
      $param_type = 'SPAM';
      break;
    default:
    $param_type = 'Other violation';
  }

  Info("Abuse type: " . $param_type);
  Info("Abuse value: " . $param_value);
  Info("Abuse message: " . $param_text);
  $report_code = report_fraudhosting($hashed_email, $hashed_ip, $param_text, $param_type, $param_value);

  // Return default xml to billing
  $xml = simplexml_load_string($default_xml_string);
  echo $xml->asXML();
  die();
}

if ($lang == 'ru') {
  $name='ФИО';
  $email='Email адрес';
  $regip='IP адрес регистрации';
  $type='Тип нарушения';
  $hint_type='Выберите тип нарушения чтобы другие провайдеры понимали что ожидать от клиента';
  $value='Уровень важности';
  $hint_value='Значение важности от 1 до 10. Где 1 - незначительно, 5 - средний уровень, 10 - критический уровень';
  $text='Сообщение нарушения';
  $hint_text='Добавьте контекст к вашей жалобе, это поможет другим провайдерам. Например, укажите содержимое abuse сообщения связанное с данным клиентом';
  $too_many_support_tickets='Слишком много запросов в поддержку';
  $chargeback='Возврат платежа';
  $stolen_card='Краденная карта';
  $fraud='Мошенничество';
  $early_cancellation='Ранний отказ от услуги';
  $dmca='DMCA жалоба';
  $excessive_server_load='Чрезмерная нагрузка на сервер';
  $public_threats='Публичные угрозы';
  $drugs_content='Контент с наркотиками';
  $sexual_exploitation='Сексуальная эксплуатация';
  $malwares='Вредоносное ПО';
  $port_scan='Сканирование портов';
  $phishing='Фишинг';
  $spam='СПАМ';
  $other='Другое нарушение';
  $hint_reload='Обновить форму';
  $hint_copylink='Скопировать ссылку на форму';
  $hint_takefavorite='Добавить в избранное меню';
  $hint_takeunfavorite='Убрать из избранного меню';
  $hint_pin='Закрепить вкладку';
  $hint_unpin='Открепить вкладку';
  $msg_ok='OK';
  $msg_cancel='Отмена';
  $title='Жалоба на клиента';
} else {
  $name='Name';
  $email='Email address';
  $regip='Register IP address';
  $type='Abuse type';
  $hint_type='Select the type of violation so that other providers understand what to expect from the client';
  $value='Severity level';
  $hint_value='Value from 1 to 10. Where 1 is insignificant, 5 is average, 10 is critical';
  $text='Abuse context';
  $hint_text='Add context to your report, this will help other providers. For example, specify the content of the abuse message related to the client';
  $too_many_support_tickets='Too many support tickets';
  $chargeback='Chargeback';
  $stolen_card='Stolen card';
  $fraud='Fraud';
  $early_cancellation='Early cancellation';
  $dmca='DMCA';
  $excessive_server_load='Excessive server load';
  $public_threats='Public threats';
  $drugs_content='Drugs content';
  $sexual_exploitation='Sexual exploitation';
  $malwares='Malwares';
  $port_scan='Port Scan';
  $phishing='Phishing';
  $spam='SPAM';
  $other='Other violation';
  $hint_reload='Reload the form';
  $hint_copylink='Copy the link to the form';
  $hint_addfavorite='Add to favorites menu';
  $hint_removefavorite='Remove from favorites menu';
  $hint_pin='Pin the tab';
  $hint_unpin='Unpin the tab';
  $msg_ok='OK';
  $msg_cancel='Cancel';
  $title='Report client';
}

// Return form
$xml = simplexml_load_string($default_xml_string);
$xml->addAttribute('func', 'fraudhosting.report');

// Add metadata
$metadata = $xml->addChild('metadata');
$metadata->addAttribute('name', 'fraudhosting.report');
$metadata->addAttribute('type', 'form');
$metadata->addAttribute('mgr', 'billmgr');

// Add form
$form = $metadata->addChild('form');
$input = $form->addChild('input');
$input->addAttribute('type', 'text');
$input->addAttribute('name', 'elid');
$input->addAttribute('readonly', 'yes');
$input->addAttribute('hidden', 'yes');

// Add hidden client_id field
$field = $form->addChild('field');
$field->addAttribute('name', 'name');
$input = $field->addChild('input');
$input->addAttribute('type', 'text');
$input->addAttribute('name', 'name');
$input->addAttribute('readonly', 'yes');

// Add client email field
$field = $form->addChild('field');
$field->addAttribute('name', 'email');
$input = $field->addChild('input');
$input->addAttribute('type', 'text');
$input->addAttribute('name', 'email');
$input->addAttribute('readonly', 'yes');

// Add cleint regip field
$field = $form->addChild('field');
$field->addAttribute('name', 'regip');
$input = $field->addChild('input');
$input->addAttribute('type', 'text');
$input->addAttribute('name', 'regip');
$input->addAttribute('readonly', 'yes');

// Add report type
$field = $form->addChild('field');
$field->addAttribute('name', 'type');
$select = $field->addChild('select');
$select->addAttribute('name', 'type');

// Add report value
$field = $form->addChild('field');
$field->addAttribute('name', 'value');
$slider = $field->addChild('slider');
$slider->addAttribute('name', 'value');
$slider->addAttribute('step', '1');
$slider->addAttribute('min', '1');
$slider->addAttribute('max', '10');

// Add report context
$field = $form->addChild('field');
$field->addAttribute('name', 'text');
$field->addAttribute('type', 'textarea');
$textarea = $field->addChild('textarea');
$textarea->addAttribute('name', 'text');
$textarea->addAttribute('rows', '10');
$textarea->addAttribute('wrap', 'off');
$textarea->addAttribute('fullwidth', 'no');
$textarea->addAttribute('readonly', 'no');
$textarea->addAttribute('required', 'yes');

// Add form buttons
$buttons = $form->addChild('buttons');
$button = $buttons->addChild('button');
$button->addAttribute('name', 'ok');
$button->addAttribute('type', 'ok');
$button = $buttons->addChild('button');
$button->addAttribute('name', 'cancel');
$button->addAttribute('type', 'cancel');

// Add report type list
$slist_data = [
  'too_many_support_tickets' => $too_many_support_tickets, 'chargeback' => $chargeback, 'stolen_card' => $stolen_card,
  'fraud' => $fraud, 'early_cancellation' => $early_cancellation, 'dmca' => $dmca,
  'excessive_server_load' => $excessive_server_load, 'public_threats' => $public_threats, 'drugs_content' => $drugs_content,
  'sexual_exploitation' => $sexual_exploitation, 'malwares' => $malwares, 'port_scan' => $port_scan,
  'phishing' => $phishing, 'spam' => $spam, 'other' => $other
];
$slist = $xml->addChild('slist');
$slist->addAttribute('name', 'type');
foreach ($slist_data as $key => $val) {
  $val = $slist->addChild('val', $val);
  $val->addAttribute('key', $key);
}

// Add messages for fields
$messages_data = [
  'name' => $name, 'email' => $email, 'regip' => $regip, 'type' => $type, 'hint_type' => $hint_type,
  'value' => $value, 'hint_value' => $hint_value, 'text' => $text, 'hint_text' => $hint_text,
  'hint_reload' => $hint_reload, 'hint_copylink' => $hint_copylink, 'hint_takefavorite' => $hint_addfavorite,
  'hint_takeunfavorite' => $hint_removefavorite, 'hint_pin' => $hint_pin, 'hint_unpin' => $hint_unpin,
  'msg_ok' => $msg_ok, 'msg_cancel' => $msg_cancel, 'title' => $title
];
$messages = $xml->addChild('messages');
$messages->addAttribute('name', 'fraudhosting.report');
foreach ($messages_data as $key => $val) {
    $msg = $messages->addChild('msg', $val);
    $msg->addAttribute('name', $key);
}

// Add form values
$name = $xml->addChild('name', $realname_value);
$email = $xml->addChild('email', $email_value);
$regip = $xml->addChild('regip', $regip_value);
$elid = $xml->addChild('elid', $param_elid);
$value = $xml->addChild('value', '1');

Info($xml->asXML());
echo $xml->asXML();